// C Codefile fuer
////////////////////////
// Snake Version 1.2a //
////////////////////////
// von Michael Ludwig und Jens Scheffler
// 2003

#include "snake.h"
#include <PalmOS.h>
#include <PalmTypes.h>

// TODO:
// *****
// -Menpunkt Pause einfgen?
// -Allgemeine Hilfe

/*************************************
*   Interne Strukturen               *
**************************************/

// Einzelner Highscore-Eintrag
typedef struct
{
    char Name[HIGHSCORE_NAME_LEN];
    int  Score;
    char Level; 
    
} HighScoreElement;

// Speicherzelle fr ein Spielfeld
typedef struct
{
    int BitmapID;
    unsigned char NachfolgerRichtung;
} FeldType;

// Struktur, um Koordinaten zu speichern
typedef struct
{
    unsigned char X;
    unsigned char Y;
} PositionType;

// Preferences-Struktur (um Spielstand zu speichern)
typedef struct 
{
                      // Koordinate des Kopfes
    PositionType      kopfPos;
                      // Koordinate des Schwanzes
    PositionType      schwanzPos;
                      // Laufrichtung der Schlange im letzten Schritt
    unsigned char     AlteRichtung;
                      // Aktuelle Laufrichtung der Schlange
    unsigned char     richtung;
                      // Inhalt des Spielfeldes
    FeldType          Feld [FELD_BREITE][FELD_HOEHE];
                      // Aktuelles Schlangenwachstum durch Fressen
    unsigned char     Wachstum;
                      // Aktuelle Lnge der Schlange
    unsigned char     Laenge;
                      // Aktuelle Punkte des laufenden Spieles
    int               Score;
                      // Aktueller Level (Negativ, wenn Game-Over)
    char              Level;
                      // Highscore-Eintrge
    HighScoreElement  HighScore[HIGHSCORE_LEN];
                      // Level, in dem ein neues Spiel gestartet wird
    char              StartLevel;
                      //Letzter eingetragener Name in der Highscore
    unsigned char     LetzterName;
                      // Kennzeichen, ob der Sound aktiviert ist
    char              SoundActive;
                      //Position des Jokers und restliche Lebensdauer des Jokers
    PositionType      Joker;
    int               JokerRestdauer;
} SnakePreferenceType;

// Definition der verschiedenen Klnge bei Snake
typedef enum SnakeSounds
{
    SoundWalk,
    SoundEat,
    SoundDie
} SnakeSounds;

/***********************************
*   Globale Variablen              *
************************************/

                    // Variablen fr geladene Bitmaps
MemPtr              bild[BILD_ANZAHL];
BitmapPtr           bildPtr[BILD_ANZAHL];

                    // Statusvariablen
long                geschwindigkeit;
long                restgeschwindigkeit;
unsigned char       HindernisAnzahl;
unsigned char       LastStepWasScore;        //Kennzeichen, ob beim letzten Schritt Punkte gemacht wurden.

                    // Struktur fr Einstellungen
SnakePreferenceType Pref;
                    // Fr Zufallszahlen
unsigned long       Seed;

                    // Zur Ereignisverarbeitung:
                    // 1, wenn Seit letztem Schritt eine Taste gedrckt wurde
unsigned char       Tastendruck;    
                    // Puffer fr 2. Richtungswechsel in einem Schritt
unsigned char       TastendruckPuffer;
                    // Verhindern, dass bei Dialog die Anwendung aus versehen
                    //    beendet wird.
unsigned char       IgnoreExit; 
                    // Variable um Menstatus abzulegen
unsigned char       MainFormAktiv;


/****************************************
*   Interne Konstanten                  *
*****************************************/
#define appFileCreator     'strt'
#define appVersionNum      0x01
#define appPrefID          0x01
#define appPrefVersionNum  0x01

// Minimale PalmOS-Version, die untersttzt wird
#define ourMinVersion   sysMakeROMVersion(2,0,0,sysROMStageRelease,0)


/********************************
*   Interne Funktionen          *
*********************************/
void AppStop(void);
void ZeigeHighScore();
void GameStart();

/***********************************************************
* Funktion:     Zufallszahl                                *
************************************************************
* Beschreibung: Bestimmt eine neue Zufallszahl             *
*                                                          *
* Parameter:    Keine                                      *
************************************************************/
unsigned char Zufallszahl()
{
    Seed=SysRandom(Seed+TimGetTicks());
    return Seed;
}

/***********************************************************
* Funktion:     DialogHandleEvent                          *
************************************************************
* Beschreibung: Verarbeitet die Events, die in allen       *
*               Dialogen auftreten. Ziel: Abfangen der     *
*               HardKeys und alles andere weiterleiten     *
*                                                          *
* Parameter:    event: Pointer auf den Event               *
************************************************************/
Boolean DialogHandleEvent(EventPtr event)
{
    if (event->eType==keyDownEvent)
    {
        switch (event->data.keyDown.chr)
        {
        case hard1Chr:
        case hard2Chr:
        case hard3Chr:
        case hard4Chr:
        case pageUpChr: 
        case pageDownChr:
        case leftArrowChr:
        case rightArrowChr:
        case upArrowChr:
        case downArrowChr:
            return true;
        }
    }
    return false;
}

/***********************************************************
* Funktion:     PlaySound                                  *
************************************************************
* Beschreibung: Spielt einen Klan fr Snake ab             *
*                                                          *
* Parameter:    ID des Klangs (Siehe SnakeSound ENUM)      *
************************************************************/
void PlaySound(int SoundID)
{
    if (Pref.SoundActive)
    {
        switch (SoundID)
        {
            case SoundWalk:
                SndPlaySystemSound(sndClick);
                break;
            case SoundEat:
                SndPlaySystemSound(sndConfirmation);
                break;
            case SoundDie:
                SndPlaySystemSound(sndWarning);
                break;
        }
    }
}

/***********************************************************
* Funktion:     ZeigeDialog                                *
************************************************************
* Beschreibung: Zeigt einen Dialog an                      *
*                                                          *
* Parameter:    IFD des Forms                              *
************************************************************/
void ZeigeDialog(int FormID)
{
    // Pointer zu den Formularen
    FormPtr MainP;
    FormPtr DialogP;
    
    // Aktuelles Formular sichern
    MainP = FrmGetActiveForm();

    // Info anzeigen
    DialogP = FrmInitForm(FormID);
    
    // Neuen EventHandler setzen (Um HardButtons abzufangen)
    FrmSetEventHandler(DialogP, DialogHandleEvent);
    
    // Warten OK-Button
    FrmDoDialog (DialogP);
    
    // Formular lschen und altes anzeigen
    FrmDeleteForm(DialogP);
    FrmSetActiveForm(MainP);
}

/***********************************************************
* Funktion:     ZeigeInfo                                  *
************************************************************
* Beschreibung: Zeigt die Informationen zum Programm an    *
*                                                          *
* Parameter:    Keine                                      *
************************************************************/
void ZeigeInfo()
{
    ZeigeDialog(FORM_INFO);
}

/***********************************************************
* Funktion:     ZeigeEinstellungen                         *
************************************************************
* Beschreibung: Zeigt den Einstellungsdialog an            *
*                                                          *
* Parameter:    Keine                                      *
************************************************************/
void ZeigeEinstellungen()
{
    // Pointer zu den Formularen
    FormPtr MainP;
    FormPtr EinstellungenP;
    
    // Aktuelles Formular sichern
    MainP = FrmGetActiveForm();

    // Info anzeigen
    EinstellungenP = FrmInitForm(FORM_EINSTELLUNGEN);
    
    // Alte Einstellungen vorsetzen
    switch (Pref.StartLevel)
    {
    case 1:
        FrmSetControlGroupSelection(EinstellungenP, EINSTELLUNGEN_GROUP_ID, EINSTELLUNGEN_PUSHBUTTON_LEVEL1);
        break;
    case 2:
        FrmSetControlGroupSelection(EinstellungenP, EINSTELLUNGEN_GROUP_ID, EINSTELLUNGEN_PUSHBUTTON_LEVEL2);
        break;
    case 3:
        FrmSetControlGroupSelection(EinstellungenP, EINSTELLUNGEN_GROUP_ID, EINSTELLUNGEN_PUSHBUTTON_LEVEL3);
        break;
    case 4:
        FrmSetControlGroupSelection(EinstellungenP, EINSTELLUNGEN_GROUP_ID, EINSTELLUNGEN_PUSHBUTTON_LEVEL4);
        break;
    case 5:
        FrmSetControlGroupSelection(EinstellungenP, EINSTELLUNGEN_GROUP_ID, EINSTELLUNGEN_PUSHBUTTON_LEVEL5);
        break;
    case 6:
        FrmSetControlGroupSelection(EinstellungenP, EINSTELLUNGEN_GROUP_ID, EINSTELLUNGEN_PUSHBUTTON_LEVEL6);
        break;
    case 7:
        FrmSetControlGroupSelection(EinstellungenP, EINSTELLUNGEN_GROUP_ID, EINSTELLUNGEN_PUSHBUTTON_LEVEL7);
        break;
    case 8:
        FrmSetControlGroupSelection(EinstellungenP, EINSTELLUNGEN_GROUP_ID, EINSTELLUNGEN_PUSHBUTTON_LEVEL8);
        break;
    }
    
    if (Pref.SoundActive)
        FrmSetControlValue(EinstellungenP, FrmGetObjectIndex(EinstellungenP, EINSTELLUNGEN_SOUND), 1);
    else
        FrmSetControlValue(EinstellungenP, FrmGetObjectIndex(EinstellungenP, EINSTELLUNGEN_SOUND), 0);

    // Neuen EventHandler setzen (Um HardButtons abzufangen
    FrmSetEventHandler(EinstellungenP, DialogHandleEvent);
    
    // Warten OK-Button
    FrmDoDialog(EinstellungenP);
    
    // Neue Einstellungen auslesen
    switch (FrmGetObjectId(EinstellungenP, FrmGetControlGroupSelection(EinstellungenP, EINSTELLUNGEN_GROUP_ID)))
    {
    case EINSTELLUNGEN_PUSHBUTTON_LEVEL1:
        Pref.StartLevel=1;
        break;
    case EINSTELLUNGEN_PUSHBUTTON_LEVEL2:
        Pref.StartLevel=2;
        break;
    case EINSTELLUNGEN_PUSHBUTTON_LEVEL3:
        Pref.StartLevel=3;
        break;
    case EINSTELLUNGEN_PUSHBUTTON_LEVEL4:
        Pref.StartLevel=4;
        break;
    case EINSTELLUNGEN_PUSHBUTTON_LEVEL5:
        Pref.StartLevel=5;
        break;
    case EINSTELLUNGEN_PUSHBUTTON_LEVEL6:
        Pref.StartLevel=6;
        break;
    case EINSTELLUNGEN_PUSHBUTTON_LEVEL7:
        Pref.StartLevel=7;
        break;
    case EINSTELLUNGEN_PUSHBUTTON_LEVEL8:
        Pref.StartLevel=8;
        break;
    }
    
    Pref.SoundActive=FrmGetControlValue(EinstellungenP, FrmGetObjectIndex(EinstellungenP, EINSTELLUNGEN_SOUND));

    // Formular lschen und altes anzeigen
    FrmDeleteForm(EinstellungenP);
    FrmSetActiveForm(MainP);
}

/********************************************************
* Funktion:     RomVersionCompatible                    *
*********************************************************
* Beschreibung: Testet, ob die minimale PalmOS-Version  *
*               vorhanden ist                           *
*                                                       *
* Parameter:    requiredVersion - Minimale Version,     *
*                                 die bentgt ist       *
*               launchFlags     - Zeichen, ob das UI    *
*                                 gestartet werden soll *
*                                                       *
* Rckgabe:     Fehlernummer oder Null, wenn PalmOS OK  *
*********************************************************/
Err RomVersionCompatible(UInt32 requiredVersion, UInt16 launchFlags)
{
    UInt32 romVersion;
    
    // Versionsnummer vom System abrufen und Prfen, ob es Kleiner ist
    FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
    if (romVersion < requiredVersion)
    {
        if ((launchFlags&(sysAppLaunchFlagNewGlobals|sysAppLaunchFlagUIApp))==(sysAppLaunchFlagNewGlobals|sysAppLaunchFlagUIApp))
        {
            // Fehlermeldung anzeigen
            FrmAlert (ROM_INCOMPATIBLE_ALERT);
            
            // PalmOS 1.0 fhrt die Anwendung erneut aus, deswegen Launcher starten
            if (romVersion<sysMakeROMVersion(2,0,0,sysROMStageRelease,0))
                AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
        }
        return (sysErrRomIncompatible);
    }
    return 0;
}

/**********************************************************
* Funktion:     MainFormInit                              *
***********************************************************
* Beschreibung: Initialisiert das Hauptformular           *
*               Unntig?!?                                *
*                                                         *
* Parameter:    command  - Menpunkt-ID                   *
*                                                         *
* Rckgabe:     Ob Punkt abgearbeitet wurde               *
***********************************************************/
void MainFormInit(FormPtr frmP)
{
}

/**********************************************************
* Funktion:     MainFormDoCommand                         *
***********************************************************
* Beschreibung: Fhrt alle Menpunkte aus                 *
*                                                         *
* Parameter:    command  - Menpunkt-ID                   *
*                                                         *
* Rckgabe:     Ob Punkt abgearbeitet wurde               *
***********************************************************/
Boolean MainFormDoCommand(UInt16 command)
{
    switch (command)
    {
    case MENUITEM_NEU:
        MenuEraseStatus(0);
        GameStart();
        return true;    
    case MENUITEM_HIGHSCORE:
        MenuEraseStatus(0);
        ZeigeHighScore();
        return true;
    case MENUITEM_EINSTELLUNGEN:
        MenuEraseStatus(0);
        ZeigeEinstellungen();
        return true;
    case MENUITEM_ABOUT:
        MenuEraseStatus(0);
        ZeigeInfo();
        return true; 
    default:
        return false;
    }
}

/***********************************************************
* Funktion:     MainFormHandleEvent                        *
************************************************************
* Beschreibung: Routine, um Ereignisse frs Hauptformular  *
*               zu verarbeiten                             *
*                                                          *
* Parameter:    eventP  - Pointer zum Event                *
*                                                          *
* Rckgabe:     TRUE, wenn das Ereignis behandelt wurde    *
*               und nicht weiter verarbeitet werden muss   *
************************************************************/
Boolean MainFormHandleEvent(EventPtr eventP)
{
    FormPtr frmP;
    
    switch (eventP->eType) 
    {
    case menuEvent:
        return MainFormDoCommand(eventP->data.menu.itemID);
    case frmOpenEvent:
        frmP = FrmGetActiveForm();
        MainFormInit(frmP);
        FrmDrawForm (frmP);
        return true;
    default:
        return false;
    }
}

/***********************************************************
* Funktion:     Get/SetBitmap                              *
************************************************************
* Beschreibung: Funktionen, um ein Bild auf das Spielfeld  *
*               zu setzen                                  *
*                                                          *
* Parameter:    Pos - Position, die zu setzen ist          *
************************************************************/
void SetBitmapXY(unsigned char PosX, unsigned char PosY, int BildID)
{
    Pref.Feld[PosX][PosY].BitmapID=BildID-BILD_ERSTES;
}
void SetBitmap(PositionType Pos, int BildID)
{
    SetBitmapXY(Pos.X, Pos.Y, BildID);
}
int GetBitmapXY(unsigned char PosX, unsigned char PosY)
{
    if (Pref.Feld[PosX][PosY].BitmapID < BILD_ERSTES)
        return Pref.Feld[PosX][PosY].BitmapID+BILD_ERSTES;
    else
        return Pref.Feld[PosX][PosY].BitmapID;
}
int GetBitmap(PositionType Pos)
{
    return GetBitmapXY(Pos.X, Pos.Y);
}

/***********************************************************
* Funktion:     Get/SetNachfolger                          *
************************************************************
* Beschreibung: Funktionen, um auf ein Spielfeld den       *
*               Nachfolger zu setzen                       *
*                                                          *
* Parameter:    Pos - Position, die zu setzen ist          *
************************************************************/
void SetNachfolgerXY(unsigned char PosX, unsigned char PosY, unsigned char Richtung)
{
    Pref.Feld[PosX][PosY].NachfolgerRichtung=Richtung;
}
void SetNachfolger(PositionType Pos, unsigned char Richtung)
{
    SetNachfolgerXY(Pos.X, Pos.Y, Richtung);
}
unsigned char GetNachfolgerXY(unsigned char PosX, unsigned char PosY)
{
    return Pref.Feld[PosX][PosY].NachfolgerRichtung;
}
unsigned char GetNachfolger(PositionType Pos)
{
    return GetNachfolgerXY(Pos.X, Pos.Y);
}

/***********************************************************
* Funktion:     PaintSnake                                 *
************************************************************
* Beschreibung: Funktion, um ein Bild auf das Spielfeld    *
*               zu zeichnen                                *
*                                                          *
* Parameter:    Pos - Position, die zu Malen ist           *
************************************************************/
void PaintSnake(PositionType Pos)
{
    WinDrawBitmap(bildPtr[GetBitmap(Pos)-BILD_ERSTES], Pos.X*RASTER+4, Pos.Y*RASTER+20);
}

/***********************************************************
* Funktion:     SavePrefs                                  *
************************************************************
* Beschreibung: Funktion, um den aktuellen Einstellungen   *
*               zu speichern                               *
*                                                          *
* Parameter:    Pos - Position, die zu Malen ist           *
************************************************************/
void SavePrefs()
{
    // Preferences Schreiben
    PrefSetAppPreferences(appFileCreator, appPrefID, appPrefVersionNum, &Pref, sizeof (Pref), true);
}

/***********************************************************
* Funktion:     ZeigeHighScore                             *
************************************************************
* Beschreibung: Zeigt die HighScore an                     *
*                                                          *
* Parameter:    Keine                                      *
************************************************************/
void ZeigeHighScore()
{
    // Pointer zu den Formularen
    FormPtr          MainP;
    FormPtr          HighScoreP;
    unsigned char    i;
    char             Ausgabe[16];
    MemHandle hnd_name;
    MemHandle hnd_punkte;
    MemHandle hnd_level;
    MemPtr  ptr_name;
    MemPtr  ptr_punkte;
    MemPtr  ptr_level;
    
    // Aktuelles Formular sichern
    MainP = FrmGetActiveForm();

    // HighScore anzeigen
    HighScoreP = FrmInitForm(FORM_HIGHSCORE);
    FrmSetActiveForm(HighScoreP);
    FrmDrawForm(HighScoreP);
    
    // berschriften schreiben
    hnd_name=   DmGetResource(strRsc,STRING_NAME);
    hnd_punkte= DmGetResource(strRsc,STRING_PUNKTE);
    hnd_level=  DmGetResource(strRsc,STRING_LEVEL);
    ptr_name=   MemHandleLock(hnd_name);
    ptr_punkte= MemHandleLock(hnd_punkte);
    ptr_level=  MemHandleLock(hnd_level);
    WinDrawChars(ptr_name,   StrLen(ptr_name)   , 3,   15);
    WinDrawChars(ptr_punkte, StrLen(ptr_punkte) , 93,  15);
    WinDrawChars(ptr_level,  StrLen(ptr_level)  , 128, 15);
    MemHandleUnlock(hnd_name);
    MemHandleUnlock(hnd_punkte);
    MemHandleUnlock(hnd_level);
    DmReleaseResource(hnd_name);
    DmReleaseResource(hnd_punkte);
    DmReleaseResource(hnd_level);
    
    // Text rausschreiben
    for (i=0;i<HIGHSCORE_LEN;i++)
    {
        // Namen schreiben
        WinDrawChars(Pref.HighScore[i].Name, StrLen(Pref.HighScore[i].Name), 5, 10*i+25);
        StrIToA(Ausgabe, Pref.HighScore[i].Score);
        WinDrawChars(Ausgabe, StrLen(Ausgabe), 95, 10*i+25);
        StrIToA(Ausgabe, Pref.HighScore[i].Level);
        WinDrawChars(Ausgabe, StrLen(Ausgabe), 130, 10*i+25);
    }

    // Neuen EventHandler setzen (Um HardButtons abzufangen
    FrmSetEventHandler(HighScoreP, DialogHandleEvent);
    
    // Warten OK-Button
    FrmDoDialog (HighScoreP);
    
    // Formular lschen und altes anzeigen
    FrmDeleteForm(HighScoreP);
    FrmSetActiveForm(MainP);
}

/***********************************************************
* Funktion:     GameOver                                   *
************************************************************
* Beschreibung: Beendet ein Spiel und setzt ggf.           *
*               die Highscore                              *
*                                                          *
* Parameter:    Keine                                      *
************************************************************/
void GameOver()
{
    FormPtr         MainP;
    FormPtr         NewHighScoreP;
    UInt16          ObjIndex;
    MemPtr          NameH;
    char*           NameP;
    UInt16          ButtonHit;
    unsigned char   i;
    
    // Schlangenkopf als Tod-Bild setzen (je nach Richtung)
    switch (Pref.richtung)
    {
    case RICHTUNG_NORD:
        SetBitmap(Pref.kopfPos, DEAD_NORD_BITMAP);
        break;
    case RICHTUNG_SUED:
        SetBitmap(Pref.kopfPos, DEAD_SUED_BITMAP);
        break;
    case RICHTUNG_WEST:
        SetBitmap(Pref.kopfPos, DEAD_WEST_BITMAP);
        break;
    case RICHTUNG_OST:
        SetBitmap(Pref.kopfPos, DEAD_OST_BITMAP);
        break;
    }
    PaintSnake(Pref.kopfPos);
    
    //Harbutton-Druck ignorieren, damit Snake nicht gleich beendet wird
    IgnoreExit = 1;
    
    // Testen, ob neue HighScore vorliegt
    if (Pref.Score>Pref.HighScore[HIGHSCORE_LEN-1].Score)
    {
        // Namen fr HighScore abfragen
        NameH = MemHandleNew (HIGHSCORE_NAME_LEN);
        NameP = MemHandleLock (NameH);
        
        // Letzten Namen vorgeben
        if (Pref.LetzterName!=HIGHSCORE_LEN)
        {
            StrCopy(NameP, Pref.HighScore[Pref.LetzterName].Name);
        }
        else
        {
            StrCopy(NameP, "");
        }
        MemPtrUnlock (NameP);
        
        // Altes Formular sichern und EingabeForm aufrufen
        MainP = FrmGetActiveForm();
        NewHighScoreP = FrmInitForm(FORM_NEW_HIGHSCORE);
        
        // Eingabefeld Fllen und aktiv setzen
        ObjIndex = FrmGetObjectIndex(NewHighScoreP, NEW_HIGHSCORE_NAME);
        FldSetTextHandle(FrmGetObjectPtr(NewHighScoreP, ObjIndex), NameH);
        FrmSetFocus(NewHighScoreP, ObjIndex);
        
        // Neuen EventHandler setzen (Um HardButtons abzufangen
        FrmSetEventHandler(NewHighScoreP, DialogHandleEvent);
    
        // Warten auf Eingabe
        ButtonHit = FrmDoDialog(NewHighScoreP);
    
        // Text sichern und Formular lschen
        FldSetTextHandle(FrmGetObjectPtr(NewHighScoreP, ObjIndex), 0);
        FrmDeleteForm(NewHighScoreP);
        FrmSetActiveForm(MainP);
        
        // Wenn OK geklickt wurde
        if (ButtonHit==NEW_HIGHSCORE_BUTTON_OK)
        {
            NameP=MemHandleLock(NameH);
    
            // Neue HighScore sichern (an die richtige Position)
            i=HIGHSCORE_LEN-1;
            
            // Alte HighScores verschieben
            while (i>0 && Pref.Score>Pref.HighScore[i-1].Score)
            {
                StrCopy(Pref.HighScore[i].Name,Pref.HighScore[i-1].Name);
                Pref.HighScore[i].Score=Pref.HighScore[i-1].Score;
                Pref.HighScore[i].Level=Pref.HighScore[i-1].Level;
                i--;
            }
            
            // Neue sichern
            StrCopy(Pref.HighScore[i].Name,NameP);
            Pref.HighScore[i].Score=Pref.Score;
            Pref.HighScore[i].Level=Pref.Level;
            
            // Letzten Eintrag als Vorgabe fr den nchsten Merken
            Pref.LetzterName=i;
            
            // Einstellungen & Highscore speichern
            SavePrefs();

            MemPtrUnlock(NameP);
            }
        
        // Handle freigen nicht vergessen!
        MemHandleFree(NameH);
    }
    else
    {
        // Bedauern anzeigen
//      FrmAlert(CrashAlert);

        ZeigeDialog(FORM_CRASH);
    }
    
    // HighScore Liste anzeigen
    ZeigeHighScore();
    
    // Status neu setzen
    geschwindigkeit=-1;
    Pref.Level=-Pref.Level;

    // Snake beenden (alt)
    //   event.eType = appStopEvent;
    //   EvtAddEventToQueue(&event);
}

/***********************************************************
* Funktion:     WriteScore                                 *
************************************************************
* Beschreibung: Schreibt die Punktzahl in die Titelleiste  *
*                                                          *
* Parameter:    Keine                                      *
************************************************************/
void WriteScore()
{
    Char Ausgabe[45]; 
    Char Temp[15];  
    MemHandle hnd_punkte;
    MemHandle hnd_level;
    MemPtr  ptr_punkte;
    MemPtr  ptr_level;
    
    // Text kopieren (1. Level)
    hnd_punkte= DmGetResource(strRsc,STRING_PUNKTE);
    hnd_level=  DmGetResource(strRsc,STRING_LEVEL);
    ptr_punkte= MemHandleLock(hnd_punkte);
    ptr_level=  MemHandleLock(hnd_level);

    StrCopy (Ausgabe, ptr_level);
    StrCat (Ausgabe, ": ");
    if (Pref.Level>=0)
        StrIToA(Temp, Pref.Level);
    else
        StrIToA(Temp, Pref.Level*-1);
    StrCat(Ausgabe, Temp);

    // Text kopieren (2. Punkte)
    StrCat(Ausgabe, "   ");
    StrCat(Ausgabe, ptr_punkte);
    StrCat(Ausgabe, ": ");
    StrIToA(Temp, Pref.Score);
    StrCat(Ausgabe, Temp);
    StrCat(Ausgabe, "      ");
    
    // Text ausgeben
    WinDrawChars(Ausgabe, StrLen(Ausgabe), 60, 2);

    MemHandleUnlock(hnd_punkte);
    MemHandleUnlock(hnd_level);
    DmReleaseResource(hnd_punkte);
    DmReleaseResource(hnd_level);
}

/***********************************************************
* Funktion:     WriteDebug                                 *
************************************************************
* Beschreibung: Schreibt die Debug-Meldungen               *
*                                                          *
* Parameter:    Integer Wert                               *
************************************************************/
/*
static int height=0;
static void WriteDebug(int var)
{
    //return;

    Char Ausgabe[20]; 
    Char Temp[10];  
    
    // Debug ausgabe
    StrCopy(Ausgabe, "Debug: ");
    StrIToA(Temp, var);
    StrCat(Ausgabe, Temp);
    StrCat(Ausgabe, "  ");
    WinDrawChars(Ausgabe, StrLen(Ausgabe), 90, 10+height%135);
    height+=10;
}
*/

/***********************************************************
* Funktion:     FruchtSetzen                               *
************************************************************
* Beschreibung: Erzeugt eine neue Frucht (Apfel/Joker) und *
*               schreibt neue Highscore                    *
*                                                          *
* Parameter:    Type: Bitmap, was gesetzt werden soll...   *
************************************************************/
void FruchtSetzen(int Type)
{
    PositionType    TempPos;
    unsigned char   Frei;
    
    // Bestimmen einer neuen Position, 
    //     bis ein freier Platz gefunden wurde
    do
    {
        // Anzahl der freien Pltze neben dem Apfel zurcksetzen
        Frei=0;
      
        TempPos.X=Zufallszahl()%FELD_BREITE;
        TempPos.Y=Zufallszahl()%FELD_HOEHE;      
        
        // Zahl der freien Felder an allen Seiten suchen
        if (TempPos.X>0             && GetBitmapXY(TempPos.X-1, TempPos.Y)==LEER_BITMAP)
            Frei++;
        if (TempPos.X<FELD_BREITE-1 && GetBitmapXY(TempPos.X+1, TempPos.Y)==LEER_BITMAP) 
            Frei++;
        if (TempPos.Y>0             && GetBitmapXY(TempPos.X, TempPos.Y-1)==LEER_BITMAP) 
            Frei++;
        if (TempPos.Y<FELD_HOEHE-1  && GetBitmapXY(TempPos.X, TempPos.Y+1)==LEER_BITMAP) 
            Frei++;
    }
    while (GetBitmap(TempPos)!=LEER_BITMAP || Frei<3);
    
    // Apfel setzen und malen
    SetBitmap(TempPos, Type);
    PaintSnake(TempPos);
    
    //Falls es der Joker war, diesen merken
    if (Type==JOKER_BITMAP)
    {
        Pref.Joker.X=TempPos.X;
        Pref.Joker.Y=TempPos.Y;
        Pref.JokerRestdauer=JOKER_DAUER;
    }
    
    // Score schreiben
    WriteScore();
}

/***********************************************************
* Funktion:     HindernisSetzen                            *
************************************************************
* Beschreibung: Erzeugt neue Hindernisse bei Level-Start   *
*                                                          *
* Parameter:    Keine                                      *
************************************************************/
void HindernisSetzen()
{
    PositionType TempPos;

    // Bestimmen einer neuen Position, 
    //     bis ein freier Platz gefunden wurde,
    //     der nicht im Startbereich der Schlange liegt
    do
    {
        TempPos.X=Zufallszahl()%FELD_BREITE;
        TempPos.Y=Zufallszahl()%FELD_HOEHE;      
    }
    while (GetBitmap(TempPos)!=LEER_BITMAP || TempPos.X==FELD_BREITE/2);
   
    // Bild setzen und malen
    SetBitmap(TempPos, HINDERNIS_BITMAP);
    PaintSnake(TempPos);
}

/***********************************************************
* Funktion:     LevelStart                                 *
************************************************************
* Beschreibung: Erzeugt alle Einstellungen fr einen       *
*               neuen Level                                *
*                                                          *
* Parameter:    Level - Nummer des Levels                  *
************************************************************/
void LevelStart(char Level)
{
    unsigned char i,j;  
    RectangleType rect;
    
    // Rahmen Zeichnen
    rect.topLeft.x = 3;
    rect.topLeft.y = 19;
    rect.extent.x  = 154;
    rect.extent.y  = 138;
    // Altes Spiel loeschen
    WinEraseRectangle(&rect, 0);
    //Rahmen zeichnen
    WinDrawRectangleFrame(2, &rect);
    
    // Initialisieren der Variablen
    
    //   Kopfposition
    Pref.kopfPos.X=FELD_BREITE/2;
    Pref.kopfPos.Y=0;
    //   Schwanzposition
    Pref.schwanzPos.X=FELD_BREITE/2;
    Pref.schwanzPos.Y=0;
    //   Startlnge
    Pref.Wachstum=WACHSTUMSRATE;
    Pref.Laenge=0;
    //   Richtung festlegen
    Pref.richtung=RICHTUNG_SUED;
    Pref.AlteRichtung=RICHTUNG_SUED;

    //   Spielfeld lschen   
    for (i=0;i<FELD_BREITE;i++)
        for (j=0;j<FELD_HOEHE;j++)
            SetBitmapXY(i, j, LEER_BITMAP);

    // Hindernisse zeichnen
    HindernisAnzahl=5+Level;
    for (i=0;i<HindernisAnzahl;i++)
    {
        HindernisSetzen();
    }

    // Apfel per Zufallsvariable setzen
    FruchtSetzen(APFEL_BITMAP);

    //   Tastendruck initialisieren
    Tastendruck=0;
           
    //   Geschwindigkeit festlegen
    geschwindigkeit=GRUND_GESCHWINDIGKEIT-Level*GESCHWINDIGKEITS_ABNAHME;
    restgeschwindigkeit=geschwindigkeit;
}

/***********************************************************
* Funktion:     redrawScreen                               *
************************************************************
* Beschreibung: Zeichnet den Hauptschirm neu, wenn         *
*               das Formular gelscht wurde.               *
*                                                          *
* Parameter:    Keine                                      *
************************************************************/
void redrawScreen()
{
    PositionType TempPos; 
    unsigned char i,j;
    RectangleType rect;

    // Rahmen Zeichnen
    rect.topLeft.x = 3;
    rect.topLeft.y = 19;
    rect.extent.x  = 154;
    rect.extent.y  = 138;
    WinDrawRectangleFrame(2, &rect);

    // Spielfeld aufbauen
    for (i=0; i<FELD_BREITE; i++)
    {
        for (j=0; j<FELD_HOEHE; j++)
        {
            TempPos.X=i;
            TempPos.Y=j;
            if (GetBitmap(TempPos)!=LEER_BITMAP)
                PaintSnake(TempPos);
        } 
    }
        
    // Punkte zeichnen
    WriteScore();
}

/***********************************************************
* Funktion:     SnakeRUN                                   *
************************************************************
* Beschreibung: Setzt die Schlange eine Position weiter    *
*                                                          *
* Parameter:    Keine                                      *
************************************************************/
void SnakeRUN()
{
    PositionType TempPos;
    
    restgeschwindigkeit = geschwindigkeit;
    
    // **************************
    // *** Schwanz bearbeiten ***
    // **************************
    //    (wenn die Schlange noch am wachsen ist)
    if (Pref.Wachstum > 0)
    {
        // Restwachstum verkleinern und Schlange verlngern
        Pref.Wachstum--;
        Pref.Laenge++;

        // Prfen, ob Level zu Ende ist
        if (Pref.Laenge>=LEVEL_WECHSEL)
        {
            // Level erhhen
            if (Pref.Level<10) 
                Pref.Level++;

            // Beenden verhindern und Meldung anzeigen
            IgnoreExit = 1;
            ZeigeDialog(FORM_NEXT_LEVEL);

            // Neuen Level starten
            LevelStart (Pref.Level);

            return;
        }
    }
    // (Wenn Schlange nicht mehr am wachsen ist, dann Schwanz einziehen)
    else
    {
        //Alte Position merken
        TempPos.X = Pref.schwanzPos.X;
        TempPos.Y = Pref.schwanzPos.Y;
        
        // Je nach Schwanzrichtung Position verndern
        switch (GetNachfolger(Pref.schwanzPos))
        {
        case RICHTUNG_NORD:
            Pref.schwanzPos.Y--;
            break;
        case RICHTUNG_SUED:
            Pref.schwanzPos.Y++;
            break;
        case RICHTUNG_WEST:
            Pref.schwanzPos.X--;
            break;
        case RICHTUNG_OST:
            Pref.schwanzPos.X++;
            break;
        }
        
        // neuen Schwanz generieren
        switch (GetNachfolger(Pref.schwanzPos))
        {
        case RICHTUNG_NORD:
            SetBitmap(Pref.schwanzPos, SCHWANZ_NORD_BITMAP);
            break;
        case RICHTUNG_SUED:
            SetBitmap(Pref.schwanzPos, SCHWANZ_SUED_BITMAP);
            break;
        case RICHTUNG_WEST:
            SetBitmap(Pref.schwanzPos, SCHWANZ_WEST_BITMAP);
            break;
        case RICHTUNG_OST:
            SetBitmap(Pref.schwanzPos, SCHWANZ_OST_BITMAP);
            break;
        }
        PaintSnake(Pref.schwanzPos);
        
        // Alte Schwanzposition lschen
        SetBitmap(TempPos, LEER_BITMAP);
        PaintSnake(TempPos);
    }
    
    // *********************************
    // *** Schlangenkoerper zeichnen ***
    // *********************************
    // Je nach aktueller Richtung
    switch (Pref.richtung)
    {
    case RICHTUNG_NORD:
        // Je nach alter Richtung, beachten, ob vorher eine Fucht verspeist wurde!
        switch (Pref.AlteRichtung)
        {
        case RICHTUNG_NORD:
            if (!LastStepWasScore)
                SetBitmap(Pref.kopfPos, KOERPER_NORD_SUED_BITMAP);
            else
                SetBitmap(Pref.kopfPos, KOERPER_NORD_SUED_BITMAP_VOLL);
            break;
        case RICHTUNG_WEST:
            if (!LastStepWasScore)
                SetBitmap(Pref.kopfPos, KOERPER_OST_NORD_BITMAP);
            else
                SetBitmap(Pref.kopfPos, KOERPER_OST_NORD_BITMAP_VOLL);
            break;
        case RICHTUNG_OST:
            if (!LastStepWasScore)
                SetBitmap(Pref.kopfPos, KOERPER_WEST_NORD_BITMAP);
            else
                SetBitmap(Pref.kopfPos, KOERPER_WEST_NORD_BITMAP_VOLL);
            break;
        }
        break;
    case RICHTUNG_SUED:
        switch (Pref.AlteRichtung)
        {
        case RICHTUNG_SUED:
            if (!LastStepWasScore)
                SetBitmap(Pref.kopfPos, KOERPER_NORD_SUED_BITMAP);
            else
                SetBitmap(Pref.kopfPos, KOERPER_NORD_SUED_BITMAP_VOLL);
            break;
        case RICHTUNG_WEST:
            if (!LastStepWasScore)
                SetBitmap(Pref.kopfPos, KOERPER_OST_SUED_BITMAP);
            else
                SetBitmap(Pref.kopfPos, KOERPER_OST_SUED_BITMAP_VOLL);
            break;
        case RICHTUNG_OST:
            if (!LastStepWasScore)
                SetBitmap(Pref.kopfPos, KOERPER_WEST_SUED_BITMAP);
            else
                SetBitmap(Pref.kopfPos, KOERPER_WEST_SUED_BITMAP_VOLL);
            break;
        }
        break;
    case RICHTUNG_WEST:
        switch (Pref.AlteRichtung)
        {
        case RICHTUNG_NORD:
            if (!LastStepWasScore)
                SetBitmap(Pref.kopfPos, KOERPER_WEST_SUED_BITMAP);
            else
                SetBitmap(Pref.kopfPos, KOERPER_WEST_SUED_BITMAP_VOLL);
            break;
        case RICHTUNG_SUED:
            if (!LastStepWasScore)
                SetBitmap(Pref.kopfPos, KOERPER_WEST_NORD_BITMAP);
            else
                SetBitmap(Pref.kopfPos, KOERPER_WEST_NORD_BITMAP_VOLL);
            break;
        case RICHTUNG_WEST:
            if (!LastStepWasScore)
                SetBitmap(Pref.kopfPos, KOERPER_WEST_OST_BITMAP);
            else
                SetBitmap(Pref.kopfPos, KOERPER_WEST_OST_BITMAP_VOLL);
            break;
        }
        break;
    case RICHTUNG_OST:
        switch (Pref.AlteRichtung)
        {
        case RICHTUNG_NORD:
            if (!LastStepWasScore)
                SetBitmap(Pref.kopfPos, KOERPER_OST_SUED_BITMAP);
            else
                SetBitmap(Pref.kopfPos, KOERPER_OST_SUED_BITMAP_VOLL);
            break;
        case RICHTUNG_SUED:
            if (!LastStepWasScore)
                SetBitmap(Pref.kopfPos, KOERPER_OST_NORD_BITMAP);
            else
                SetBitmap(Pref.kopfPos, KOERPER_OST_NORD_BITMAP_VOLL);
            break;
        case RICHTUNG_OST:
            if (!LastStepWasScore)
                SetBitmap(Pref.kopfPos, KOERPER_WEST_OST_BITMAP);
            else
                SetBitmap(Pref.kopfPos, KOERPER_WEST_OST_BITMAP_VOLL);
            break;
        }
        break;
    }
    PaintSnake(Pref.kopfPos);
    
    // ******************************
    // *** Kopf neu positionieren ***
    // ******************************
    // Nachfolgeposition fr alten Kopf setzen
    //    (Damit Schwanz folgen kann!)
    SetNachfolger(Pref.kopfPos, Pref.richtung);
    
    // Kopf nach Richtung neu setzen und auf Grenze berprfen
    switch (Pref.richtung)
    {
    case RICHTUNG_NORD:
        if (Pref.kopfPos.Y>0)
        {
            Pref.kopfPos.Y--;
        }
        else
        {
            PlaySound(SoundDie);
            GameOver();
            return;
        }
        break;
    case RICHTUNG_SUED:
        if (Pref.kopfPos.Y<FELD_HOEHE-1)
        {
            Pref.kopfPos.Y++;
        }
        else
        {
            PlaySound(SoundDie);
            GameOver();
            return;
        }
        break;
    case RICHTUNG_WEST:
        if (Pref.kopfPos.X>0)
        {
            Pref.kopfPos.X--;
        }
        else
        {
            PlaySound(SoundDie);
            GameOver();
            return;
        }
        break;
    case RICHTUNG_OST:
        if (Pref.kopfPos.X<FELD_BREITE-1)
        {
            Pref.kopfPos.X++;
        }
        else
        {
            PlaySound(SoundDie);
            GameOver();
            return;
        }
        break;
    }

    // Sound abspielen je nach nchstem Schritt
    switch (GetBitmap(Pref.kopfPos))
    {
        case APFEL_BITMAP:
        case JOKER_BITMAP:
            PlaySound(SoundEat);
            break;
        case LEER_BITMAP:
            PlaySound(SoundWalk);
            break;
        default:
            PlaySound(SoundDie);
            break;
    }
    
    // Prfen, ob neue Kopfposition auf Apfel oder Joker zeigt unf ggf Score erhhen
    if (GetBitmap(Pref.kopfPos) == APFEL_BITMAP)
    {
        // Score erhhen
        Pref.Score+=3*Pref.Level;
        // Wachstum setzen
        Pref.Wachstum+=WACHSTUMSRATE;
        // Neuen Apfel setzen
        FruchtSetzen(APFEL_BITMAP);
        // Alten Apfel lschen, damit der Kopf richtig gesetzt werden kann
        SetBitmap(Pref.kopfPos, LEER_BITMAP);
        // Merken, dass wir gerade was gefressen haben und Schlangenkrper danach "gefllt" ist!
        LastStepWasScore=1;
    }
    else if (GetBitmap(Pref.kopfPos) == JOKER_BITMAP)
    {
        // Score erhhen
        Pref.Score+=3*Pref.Level*JOKER_PUNKTE_FAKTOR;
        // Wachstum setzen
        Pref.Wachstum+=WACHSTUMSRATE;
        // Jokerinformation lschen
        Pref.Joker.X=255;
        Pref.Joker.Y=255;
        Pref.JokerRestdauer=-1;
        WriteScore();
        // Joker lschen, damit der Kopf richtig gesetzt werden kann
        SetBitmap(Pref.kopfPos, LEER_BITMAP);
        // Merken, dass wir gerade was gefressen haben und Schlangenkrper danach "gefllt" ist!
        LastStepWasScore=1;
    }
    else
    {
        // Merken, dass wir gerade was gefressen haben und Schlangenkrper danach "gefllt" ist!
        LastStepWasScore=0;
    }
    
    // Auf Crash pruefen
    //    (Wenn Platz fr neuen Kopf frei ist)
    if (GetBitmap(Pref.kopfPos)==LEER_BITMAP)  // kein Crash
    {   
        // Kopf Richtung mit richtigem Bitmap belegen
        switch (Pref.richtung)
        {
        case RICHTUNG_NORD:
            // Jeweils prfen, ob auf nchstem Feld etwas liegt und Maul ffnen
            if (Pref.kopfPos.Y-1>=0 && GetBitmapXY(Pref.kopfPos.X, Pref.kopfPos.Y-1) != LEER_BITMAP)
                SetBitmap(Pref.kopfPos, KOPF_ESSEN_NORD_BITMAP);
            else
                SetBitmap(Pref.kopfPos, KOPF_NORD_BITMAP);
            break;
        case RICHTUNG_SUED:
            if (Pref.kopfPos.Y+1<FELD_HOEHE && GetBitmapXY(Pref.kopfPos.X, Pref.kopfPos.Y+1) != LEER_BITMAP)
                SetBitmap(Pref.kopfPos, KOPF_ESSEN_SUED_BITMAP);
            else
                SetBitmap(Pref.kopfPos, KOPF_SUED_BITMAP);
            break;
        case RICHTUNG_WEST:
            if (Pref.kopfPos.X-1>=0 && GetBitmapXY(Pref.kopfPos.X-1, Pref.kopfPos.Y) != LEER_BITMAP)
                SetBitmap(Pref.kopfPos, KOPF_ESSEN_WEST_BITMAP);
            else
                SetBitmap(Pref.kopfPos, KOPF_WEST_BITMAP);
            break;
        case RICHTUNG_OST:
            if (Pref.kopfPos.X+1<FELD_BREITE && GetBitmapXY(Pref.kopfPos.X+1, Pref.kopfPos.Y) != LEER_BITMAP)
                SetBitmap(Pref.kopfPos, KOPF_ESSEN_OST_BITMAP);
            else
                SetBitmap(Pref.kopfPos, KOPF_OST_BITMAP);
            break;
        }
        // Kopf zeichnen
        PaintSnake(Pref.kopfPos);
    }
    // Ansonsten war an der Kopfposition ein Hindernis = TOD!
    else
    {
        GameOver();
        return;
    }
       
    // Die Aktuelle Richtung in AlteRichtung speichern
    // Fuer das Zeichnen des Koerpers benoetigt
    Pref.AlteRichtung = Pref.richtung;
    
    //  TastendruckPuffer verarbeiten
    if (Tastendruck > 0)
    {
        if (Tastendruck > 1)
        {
            Pref.richtung = TastendruckPuffer;
        }  
        Tastendruck--;
    }

    // Joker entfernen, falls abgelaufen
    if (Pref.JokerRestdauer==0)
    {
        SetBitmap(Pref.Joker, LEER_BITMAP);
        PaintSnake(Pref.Joker);
        Pref.Joker.X=255;
        Pref.Joker.Y=255;
        Pref.JokerRestdauer=-1;
    }
    // Sonst Lebensdauer verringern...
    else if (Pref.JokerRestdauer>0)
        Pref.JokerRestdauer--;
    // Potentiell neuen Joker erzeugen...
    else
        if (0==(Zufallszahl()%JOKER_HAEUFIGKEIT))
            FruchtSetzen(JOKER_BITMAP);
}

/***********************************************************
* Funktion:     GameStart                                  *
************************************************************
* Beschreibung: Startet ein neues Spiel                    *
*                                                          *
* Parameter:    Keine                                      *
************************************************************/
void GameStart()
{
    Pref.Level = Pref.StartLevel;
    Pref.Score = 0;
    LevelStart (Pref.Level);
    Pref.Joker.X = 255;
    Pref.Joker.Y = 255;
    Pref.JokerRestdauer = -1;    
}

/********************************************************
* Funktion:     AppHandleEvent                          *
*********************************************************
* Beschreibung: This routine loads form resources and   *
*               set the event handler for the form      *
*               loaded                                  *
*                                                       *
* Parameter:    eventP - Pointer auf einen Event        *
*                                                       *
* Rckgabe:     TRUE, wenn es nicht weiter behandelt    *
*               werden muss                             *
*********************************************************/
Boolean AppHandleEvent( EventPtr eventP)
{
    UInt16 formId;
    FormPtr frmP;

    // Wenn ein Formular angezeigt werden soll
    if (eventP->eType == frmLoadEvent)
    {
        // Form laden
        formId = eventP->data.frmLoad.formID;
        frmP = FrmInitForm(formId);
        FrmSetActiveForm(frmP);
        
        // Set the event handler for the form.  The handler of the currently
        // active form is called by FrmHandleEvent each time is receives an
        // event.
        switch (formId)
        {
        case FORM_MAIN:
            FrmSetEventHandler(frmP, MainFormHandleEvent);
            break;
        default:
            //  ErrFatalDisplay("Invalid Form Load Event");
            break;  
        }
        return true;
    }
    
    // Zeit ist abgelaufen und die Schlange "geht" ein Feld weiter
    if (eventP->eType == nilEvent)
    {
        // Wenn das Spiel am Laufen ist und ein SPielzug ntig wre
        if (Pref.Level>0 && MainFormAktiv==1 && restgeschwindigkeit<=0)
        {
            SnakeRUN();
            return true;
        }
    }
    return false;
}

/********************************************************
* Funktion:     RichtungswechselHandler                 *
*********************************************************
* Beschreibung: Funtion zu Behandeln von Richtungs-     *
*               wechseln.                               *
*                                                       *
* Parameter:    richtung - angegebene Richtung, die     *
*               vom Anwender signalisiert wurde         *
*                                                       *
* Rckgabe:     TRUE, wenn die Richtung verarbeitet     *
*               wurde                                   *
*********************************************************/
unsigned char RichtungswechselHandler(unsigned char richtung)
{
    // Entscheiden, je nach anzahl der Testendrcke seit
    //    letztem Spielzug
    switch (Tastendruck)
    {
        // Erster Tastendruck
        case 0:     
        // Wenn Taste einmal gedrckt wurde
        case 1:
            switch (richtung)
            {
                case RICHTUNG_WEST:
                    if (Pref.richtung != RICHTUNG_OST && Pref.richtung != RICHTUNG_WEST)
                    {
                        if (Tastendruck==0)
                            Pref.richtung = RICHTUNG_WEST;
                        else
                            TastendruckPuffer = RICHTUNG_WEST;
                        Tastendruck++;
                    }
                    return true;
                case RICHTUNG_OST:
                    if (Pref.richtung != RICHTUNG_WEST && Pref.richtung != RICHTUNG_OST)
                    {
                        if (Tastendruck==0)
                            Pref.richtung = RICHTUNG_OST;
                        else
                            TastendruckPuffer = RICHTUNG_OST;
                        Tastendruck++;
                    }
                    return true;
                case RICHTUNG_NORD:
                    if (Pref.richtung != RICHTUNG_SUED && Pref.richtung != RICHTUNG_NORD)
                    {
                        if (Tastendruck==0)
                            Pref.richtung = RICHTUNG_NORD;
                        else
                            TastendruckPuffer = RICHTUNG_NORD;
                        Tastendruck++;
                    }
                    return true;
                case RICHTUNG_SUED:
                    if (Pref.richtung != RICHTUNG_NORD && Pref.richtung != RICHTUNG_SUED)
                    {
                        if (Tastendruck==0)
                            Pref.richtung = RICHTUNG_SUED;
                        else
                            TastendruckPuffer = RICHTUNG_SUED;
                        Tastendruck++;
                    }
                    return true;
                default:
                    return true;
            }
        //Ansonsten wurden mehr als 2 mal Richtung gewechselt
        default:
            return true;
    }
}

/********************************************************
* Funktion:     Richtungswechsel                        *
*********************************************************
* Beschreibung: Rountine zum Behandeln von Tastendrcken*
*               um die Richtung neu zu setzen           *
*               (Muss in dem Event-Loop vor den System- *
*               test getestet werden, damit nicht       *
*               Kalender gestartet werden kann          *
*                                                       *
* Parameter:    eventP - Pointer auf einen Event        *
*                                                       *
* Rckgabe:     TRUE, wenn der Tastendruck verarbeitet  *
*               wurde                                   *
*********************************************************/
unsigned char Richtungswechsel(EventType *event)
{
    // Prfen, ob ein Tastendruck erfolgt ist
    switch (event->eType)
    {
        case keyDownEvent:
            // KeyDown Events behandeln
            switch (event->data.keyDown.chr)
            {
                case hard1Chr:
                case hard2Chr:
                case leftArrowChr:
                    return RichtungswechselHandler(RICHTUNG_WEST);
                case hard3Chr:
                case hard4Chr:
                case rightArrowChr:
                    return RichtungswechselHandler(RICHTUNG_OST);
                case pageUpChr: 
                case upArrowChr:
                    return RichtungswechselHandler(RICHTUNG_NORD);
                case pageDownChr:
                case downArrowChr:
                    return RichtungswechselHandler(RICHTUNG_SUED);
                default:
                    return false;
            }
        case penDownEvent:
            // Grafitti behandeln...
            if (event->screenY > 12 && event->screenY <= 160) // Menleiste und Graffiti nicht behandeln
            {
                if (event->screenX > event->screenY)
                {
                    if ((SCREEN_WIDTH-event->screenX) > event->screenY)
                        RichtungswechselHandler(RICHTUNG_NORD);
                    else
                        RichtungswechselHandler(RICHTUNG_OST);
                }
                else
                {
                    if ((SCREEN_WIDTH-event->screenX) > event->screenY)
                        RichtungswechselHandler(RICHTUNG_WEST);
                    else
                        RichtungswechselHandler(RICHTUNG_SUED);
                }
            }
            return false;
        default:
            return false;
    }
}

/********************************************************
* Funktion:     AppEventLoop                            *
*********************************************************
* Beschreibung: Verarbeitet die Nachrichten vom System  *
*                                                       *
* Parameter:    Keine                                   *
*********************************************************/
void AppEventLoop(void)
{
    UInt16 error;
    EventType event;
    UInt32 Ticks;
    
    do 
    {
        // Prfen, ob im vorigen durchlauf eine Stopp-Meldung war und
        //    Ignorierien-Status zurcksetzen
        if (event.eType == appStopEvent)
        {
            IgnoreExit = 0;
        }

        // aktuelle Zeit speichern
        Ticks = TimGetTicks();

        // Meldung abwarten
        EvtGetEvent(&event, restgeschwindigkeit);

        // zeitdifferenz bestimmen und restzeit fr Spielzug berechnen
        restgeschwindigkeit-=TimGetTicks() - Ticks;
        
        // auf Richtungstatsten prfen
        //   (nur, wenn aktuell ein Spiel luft)
        if (Pref.Level < 0 || !Richtungswechsel(&event))
        {
            // Prfen, ob wir in das Men springen
            if (event.eType==winExitEvent && event.data.winExit.exitWindow==(WinHandle)FrmGetFormPtr(FORM_MAIN))
            {
                MainFormAktiv=0;
            }
            if (event.eType==winEnterEvent && event.data.winEnter.enterWindow==(WinHandle)FrmGetFormPtr(FORM_MAIN))
            {
                MainFormAktiv=1;
            }
        
            // Sonst das System die Nachricht behandlen lassen
            if (! SysHandleEvent(&event))
            {
                // Sonst auf Menevent prfen
                if (! MenuHandleEvent(0, &event, &error))
                {
                    // Sonst Applikation es behandeln lassen
                    if (! AppHandleEvent(&event))
                    {
                        // Restleiche Nachrochten vom Formular verarbeiten
                        FrmDispatchEvent(&event);
                    }
                }
            }
        }

        // Wenn das Main-Form aufgerufen wurde, den Bildschirm neu zeichnen
        if (event.eType==frmOpenEvent && event.data.frmLoad.formID==FORM_MAIN)
        {
            redrawScreen();
        }
    } while (IgnoreExit == 1 || event.eType != appStopEvent);
}


/********************************************************
* Funktion:     AppStart                                *
*********************************************************
* Beschreibung: Besorgt den STart des Spiels            *
*               und ld Einstellungen                   *
*                                                       *
* Parameter:    Keine                                   *
*                                                       *
* Rckgabe:     Fehlernummer                            *
*********************************************************/
Err AppStart(void)
{
    UInt16 PrefSize;
    unsigned char i;
    
    // Bennden Nicht Ignorieren
    IgnoreExit = 0;
    
    // Zufallszahlen initialisieren
    Seed = TimGetTicks();
    
    // Alle Bilder laden
    for (i=0;i<BILD_ANZAHL;i++)
    {
        bild[i]    = DmGetResource(bitmapRsc, BILD_ERSTES + i);
        bildPtr[i] = MemHandleLock(bild[i]);
    }
    
    // Einstellungen laden
    PrefSize = sizeof (Pref);
    if (PrefGetAppPreferences(appFileCreator, appPrefID, &Pref, &PrefSize, true)==noPreferenceFound)
    {
        // Wenn keine Einstellungen gefunden wurden...
        
        // ... Einstellungen setzen
        Pref.StartLevel=STANDARD_START_LEVEL;
        
        // ... HighScore initialisieren
        //         Platz 1
        StrCopy(Pref.HighScore[0].Name,"Michael");
        Pref.HighScore[0].Score=5374;
        Pref.HighScore[0].Level=9;
        //         Platz 2
        StrCopy(Pref.HighScore[1].Name,"Jens");
        Pref.HighScore[1].Score=4380;
        Pref.HighScore[1].Level=8;
        //         Platz 3
        StrCopy(Pref.HighScore[2].Name,"Thorsten");
        Pref.HighScore[2].Score=3130;
        Pref.HighScore[2].Level=7;
        //         Platz 4
        StrCopy(Pref.HighScore[3].Name,"Simone");
        Pref.HighScore[3].Score=2565;
        Pref.HighScore[3].Level=8;
        //         Platz 5
        StrCopy(Pref.HighScore[4].Name,"Klaus");
        Pref.HighScore[4].Score=1803;
        Pref.HighScore[4].Level=5;
        //         Platz 6
        StrCopy(Pref.HighScore[5].Name,"Peter");
        Pref.HighScore[5].Score=1049;
        Pref.HighScore[5].Level=3;
        //         Platz 7
        StrCopy(Pref.HighScore[6].Name,"Alexander");
        Pref.HighScore[6].Score=673;
        Pref.HighScore[6].Level=4;
        //         Platz 8
        StrCopy(Pref.HighScore[7].Name,"Steffi");
        Pref.HighScore[7].Score=3;   //305;
        Pref.HighScore[7].Level=1;
        
        // Letzten Highscore Namenseintrag initialisieren
        Pref.LetzterName=HIGHSCORE_LEN;
        
        // ... und dann neues Spiel starten
        GameStart();
    }
    else
    {   
        // Ansonsten Bildschirm wieder-hertichten
        //redrawScreen();
            
        //Geschwindigkeit festlegen
        geschwindigkeit = GRUND_GESCHWINDIGKEIT-Pref.Level*7;
        restgeschwindigkeit = geschwindigkeit;  
    }
    
    //Hauptfenster als "aktiv" markieren
    MainFormAktiv=1;
    LastStepWasScore=0;
    return 0;
}

/********************************************************
* Funktion:     AppStop                                 *
*********************************************************
* Beschreibung: Sichert alle relevanten Informationen   *
*               vor dem Beenden                         *
*                                                       *
* Parameter:    Keine                                   *
*********************************************************/
void AppStop(void)
{   
    int i;
    
    SavePrefs();
    
    //Bilder freigeben
    for (i=0;i<BILD_ANZAHL;i++)
    {
        MemHandleUnlock(bild[i]);
    }
}

/********************************************************
* Funktion:     SnakePilotMain                          *
*********************************************************
* Beschreibung: Einstiegspunkt fr das Spiel            *
*                                                       *
* Parameter:    cmd - Launch Code                       *
*               cmdPB - Pointer mit Launch Code Struktur*
*               launchFlags - Extra Info                *
*                                                       *
* Rckgabe:     Ergebnis der Anwendung                  *
*********************************************************/
UInt32 SnakePilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    Err error;
    
    // PalmOS-Version berprfen
    error = RomVersionCompatible(ourMinVersion, launchFlags);
    if (error) return error;
    
    switch (cmd)
    {
    case sysAppLaunchCmdNormalLaunch:
        // Einstellungen laden
        error = AppStart();
        if (error) return error;
        
        // Formular anzeigen
        FrmGotoForm(FORM_MAIN);

        // Event-Loop starten
        AppEventLoop();

        // Einstellungen sichern
        AppStop();
        break;
    default:
        break;
    }
    return 0;
}

/********************************************************
* Funktion:     PilotMain                               *
*********************************************************
* Beschreibung: Einstiegspunkt fr das PalmOS           *
*                                                       *
* Parameter:    cmd - Launch Code                       *
*               cmdPB - Pointer mit Launch Code Struktur*
*               launchFlags - Extra Info                *
*                                                       *
* Rckgabe:     Ergebnis der Anwendung                  *
*********************************************************/
UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    return SnakePilotMain(cmd, cmdPBP, launchFlags);
}
